/**
 * eobjects.org MetaModel
 * Copyright (C) 2010 eobjects.org
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution; if not, write to:
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301  USA
 */

package org.eobjects.metamodel.jdbc.dialects;

import org.eobjects.metamodel.jdbc.JdbcDataContext;
import org.eobjects.metamodel.query.FromItem;
import org.eobjects.metamodel.query.Query;
import org.eobjects.metamodel.schema.ColumnType;
import org.eobjects.metamodel.schema.Schema;
import org.eobjects.metamodel.schema.Table;

public class PostgresqlQueryRewriter extends DefaultQueryRewriter implements IQueryRewriter {

	public PostgresqlQueryRewriter(JdbcDataContext dataContext) {
		super(dataContext);
	}

	@Override
	public boolean isFirstRowSupported() {
		return true;
	}

	@Override
	public boolean isMaxRowsSupported() {
		return true;
	}

	@Override
	public ColumnType getColumnType(int jdbcType, String nativeType, Integer columnSize) {
		if ("bool".equals(nativeType)) {
			// override the normal behaviour of postgresql which maps "bool" to
			// a BIT.
			return ColumnType.BOOLEAN;
		}
		return super.getColumnType(jdbcType, nativeType, columnSize);
	}
	
	@Override
	public String rewriteColumnType(ColumnType columnType) {
		if (columnType == ColumnType.BLOB) {
			return "bytea";
		}
		return super.rewriteColumnType(columnType);
	}

	/**
	 * If the Max rows and/or First row property of the query is set, then we
	 * will use PostgreSQLs LIMIT and OFFSET functions.
	 */
	@Override
	public String rewriteQuery(Query query) {
		String queryString = super.rewriteQuery(query);
		final Integer maxRows = query.getMaxRows();
		if (maxRows != null) {
			queryString = queryString + " LIMIT " + maxRows;
		}

		final Integer firstRow = query.getFirstRow();
		if (firstRow != null) {
			queryString = queryString + " OFFSET " + firstRow;
		}
		return queryString;
	}

	@Override
	protected String rewriteFromItem(Query query, FromItem item) {
		String result = super.rewriteFromItem(query, item);
		Table table = item.getTable();
		if (table != null) {
			Schema schema = table.getSchema();
			if (schema != null) {
				String schemaName = schema.getName();
				if (schemaName != null) {
					result = result.replaceFirst(schemaName, '\"' + schema.getName() + '\"');
				}
			}
		}
		return result;
	}
}